/***************************************************************************
 *   Copyright (C) 2005 by Yuri Vatis   																								*
 *   vatis@tnt.uni-hannover.de																														*
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef CONTROLBAR_H
#define CONTROLBAR_H

#include <QWidget>
#include <QToolBox>
#include <QGroupBox>
#include <QTabWidget>
//#include "mainwindow.h"

class MainWindow;
class QPushButton;
class QComboBox;
class QRadioButton;
class QLabel;
class QSpinBox;
class QSlider;
class QLineEdit;

class ControlBarWidget : public QWidget {
	Q_OBJECT
	public:
		ControlBarWidget (QWidget *parent = 0, Qt::WFlags f = 0 );
		void createActions();
		void setFrameNrLabel(int fr_nr);
		void setFrameNrRange(int fr_nr);
		bool getLoopStatus();
		int	 getFrequency();
		void setFrequency(int);
		void setStopped();
	signals:
		void loopStatusChanged(bool);
	public slots:
		void setLoop();
		void setPlayPause(bool play);
		void setFrameNrSlider(int fr_nr);
		void setFrameNrSpinBox(int fr_nr);
		void frequencySet(int);
	private:
		void init();
		void initSignals();
		QLabel 			*frameNrLabel;	//shows total number of frames
		QSlider 		*currentFrameNrSlider;
		QSpinBox 		*currentFrameNrSpinBox;
		QPushButton	*stepBackPushButton;
		QPushButton	*stepForwPushButton;
		QPushButton	*playPushButton;
		QPushButton	*loopPushButton;
		QPushButton	*stopPushButton;
		QComboBox   *frequencyComboBox;
		QWidget			*parent;

		bool loop; 							// indicates, if loop set
		int  current_frame_nr;
		int frequencyValue[10];
		int frequency;
};

class VideoSizeBox: public QWidget
{
	Q_OBJECT
	public:
		VideoSizeBox(QWidget *parent = 0, Qt::WFlags flags=0);
		int getVideoSizeX();
		int getVideoSizeY();
		int getCurrentButton();
		void setCurrentButton(int, QString, QString);
		QString getQLineEditX();
		QString getQLineEditY();
	private slots:
		void setVideoSize();
	private:
//		void setGroupBox();
		void setButtons();
//		QGroupBox *sizeGroupBox;

		QRadioButton *QCIFButton;
		QRadioButton *CIFButton;
		QRadioButton *VGAButton;
		QRadioButton *SDTVButton;
		QRadioButton *HDTVButton;
    QRadioButton *SHDTVButton;
		QRadioButton *customButton;
		QLineEdit    *sizeXLineEdit;
		QLineEdit    *sizeYLineEdit;
		QLabel       *XLabel;
		QLabel       *YLabel;

		int sizeX;
		int sizeY;
};

class VideoFormatBox: public QWidget
{
	Q_OBJECT
	public:
		VideoFormatBox(QWidget *parent = 0, Qt::WFlags flags=0);
		int getVideoFormat();
		int getCurrentButton ();
		void setCurrentButton (int);
	private slots:
		void setVideoFormat();
	private:
//		void setGroupBox();
		void setButtons();
//		QGroupBox *formatGroupBox;

		QRadioButton *format400Button;
		QRadioButton *format420Button;
		QRadioButton *format422Button;
		QRadioButton *format444Button;

		int videoFormat;
};

class VideoSizeFormatTabWidget: public QTabWidget{
	Q_OBJECT
	public:
		VideoSizeFormatTabWidget(QWidget *parent = 0);
		~VideoSizeFormatTabWidget();
		VideoFormatBox   *videoFormat;
		VideoSizeBox     *videoSize;
};

class ControlToolBox: public QToolBox {
	Q_OBJECT
	public:
		ControlToolBox (QWidget *parent = 0, Qt::WFlags f = 0 );
		~ControlToolBox();

		void showSettings();
		void setTotalFrameNr(int);
		void openEvent(int);
		void setSliderValue(int);
		void setCurrentFrameNr(int);
		void setStopped();
		void restoreState();
		void setButtons(int sX, int sY, int format, bool loop, int freq);
		int	 getFrequency();
		int	 getSizeX();
		int	 getSizeY();
		int  getFormat();
		bool getLoopStatus();
		void setPlayPause(bool);
	public slots:
		void storeState();
		void okButtonClicked();
		void cancelButtonClicked();
	signals:
		void frequencyChanged();
		void loadFilesEmitted();
		void controlToolBoxClosed();
		void setLoopEmitted();
		void showNextEmitted();
		void showPreviousEmitted();
		void stopVideoEmitted();
		void playVideoEmitted();
		void sliderEmitted(int);
	protected:
		virtual void closeEvent(QCloseEvent * e );
	private:
		QPushButton *okButton;
		QPushButton *cancelButton;
		ControlBarWidget *controlBar;
		QWidget  *tabWidget;
		VideoSizeFormatTabWidget *videoSizeFormat;
		int currentSizeButton;
		int currentFormatButton;
		QString sizeXString;
		QString sizeYString;
		bool okClicked;
};

#endif
