/***************************************************************************
 *   Copyright (C) 2005 by Yuri Vatis   																	 *
 *   vatis@tnt.uni-hannover.de																						 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "controlbar.h"
#include "global.h"
#include "mainwindow.h"

#include <QSize>
#include <QRegExpValidator>
#include <QPushButton>
#include <QComboBox>
#include <QRadioButton>
#include <QLabel>
#include <QSpinBox>
#include <QSlider>
#include <QLineEdit>

#include <QCloseEvent>
#include <QString>

ControlBarWidget::ControlBarWidget( QWidget *parent, Qt::WFlags f ): QWidget(parent, f)
{
//	this->setFixedSize(CONTROLBAR_SIZE_X, CONTROLBAR_SIZE_Y);
	this->setWindowFlags(Qt::WindowTitleHint);
//	this->setWindowTitle("Control Bar");
	this->parent=parent;
	init();
	initSignals();
}

void ControlBarWidget::init()
{
	const QSize qs(20,20);
	frameNrLabel 		= new QLabel( "0 frames.",this);
	currentFrameNrSlider 	= new QSlider(Qt::Horizontal,this);
	currentFrameNrSpinBox   = new QSpinBox(this);
	stepBackPushButton 	= new QPushButton(this);
	stepForwPushButton 	= new QPushButton(this);
	playPushButton 		= new QPushButton(this);
	stopPushButton 		= new QPushButton(this);
	loopPushButton 		= new QPushButton(this);
	frequencyComboBox 	= new QComboBox(this);

	currentFrameNrSlider ->setRange(0,0);
	currentFrameNrSpinBox->setRange(0,0);

	frameNrLabel		->setGeometry	( 50,10, 90,15);
	currentFrameNrSlider	->setGeometry	(  7,24,130,20);
	currentFrameNrSpinBox	->setGeometry	(147,24, 50,20);
	stepBackPushButton	->setGeometry	(  3,60, 26,26);
	playPushButton		->setGeometry	( 30,60, 26,26);
	stepForwPushButton	->setGeometry	( 57,60, 26,26);
	stopPushButton		->setGeometry	( 84,60, 26,26);
	loopPushButton		->setGeometry	(111,60, 26,26);
	frequencyComboBox	->setGeometry	(140,60, 70,26);

	stepBackPushButton->setIcon(QIcon(":/images/stepBack.png"));
	playPushButton->setIcon(QIcon(":/images/play.png"		));
	stepForwPushButton->setIcon(QIcon(":/images/stepForw.png"));
	stopPushButton->setIcon(QIcon(":/images/stop.png"		));
	loopPushButton->setIcon(QIcon(":/images/loopUP.png"	));

	stepBackPushButton->setIconSize(qs);
	playPushButton->setIconSize(qs);
	stepForwPushButton->setIconSize(qs);
	stopPushButton->setIconSize(qs);
	loopPushButton->setIconSize(qs);

	frequencyComboBox->addItem(" 1 Hz");
	frequencyComboBox->addItem(" 2 Hz");
	frequencyComboBox->addItem(" 5 Hz");
	frequencyComboBox->addItem("10 Hz");
	frequencyComboBox->addItem("15 Hz");
	frequencyComboBox->addItem("20 Hz");
	frequencyComboBox->addItem("25 Hz");
	frequencyComboBox->addItem("30 Hz");
	frequencyComboBox->addItem("50 Hz");
	frequencyComboBox->addItem("60 Hz");
	frequencyComboBox->setCurrentIndex(7);

	loop = false;
	current_frame_nr = 0;
	frequency = 30;
	
	frequencyValue[0] =  1;
	frequencyValue[1] =  2;
	frequencyValue[2] =  5;
	frequencyValue[3] = 10;
	frequencyValue[4] = 15;
	frequencyValue[5] = 20;
	frequencyValue[6] = 25;
	frequencyValue[7] = 30;
	frequencyValue[8] = 50;
	frequencyValue[9] = 60;
}

void ControlBarWidget::initSignals()
{
	connect(loopPushButton,         SIGNAL(clicked()),                this, 	SLOT(setLoop())	);
	connect(this,                   SIGNAL(loopStatusChanged(bool)),  parent,	SIGNAL(setLoopEmitted()));
	connect(currentFrameNrSlider,   SIGNAL(valueChanged(int)),	      this, 	SLOT(setFrameNrSlider(int)));
	connect(currentFrameNrSpinBox,  SIGNAL(valueChanged(int)),        this, 	SLOT(setFrameNrSpinBox(int)));
	connect(stepForwPushButton,     SIGNAL(clicked()),		            parent, SIGNAL(showNextEmitted()));
	connect(stepBackPushButton,     SIGNAL(clicked()),                parent, SIGNAL(showPreviousEmitted()));
	connect(stopPushButton,	        SIGNAL(clicked()),                parent,	SIGNAL(stopVideoEmitted()));
	connect(currentFrameNrSlider,   SIGNAL(valueChanged(int)),        parent, SIGNAL(sliderEmitted(int)));
	connect(frequencyComboBox,      SIGNAL(activated(int)),		        this, 	SLOT(frequencySet(int))	);
	connect(frequencyComboBox,      SIGNAL(activated(int)),		        parent,	SIGNAL(frequencyChanged()));
	connect(playPushButton,         SIGNAL(clicked()),                parent,	SIGNAL(playVideoEmitted()));	
}


void ControlBarWidget::setFrameNrRange(int fr_nr)
{
	if(fr_nr > 0)
	{
		currentFrameNrSlider ->setRange(1,fr_nr);
		currentFrameNrSpinBox->setRange(1,fr_nr);
	}
	else
	{
		currentFrameNrSlider ->setRange(0,0);
		currentFrameNrSpinBox->setRange(0,0);
	}
}

void ControlBarWidget::setFrameNrLabel(int fr_nr)
{
	QString str;
	str.setNum(fr_nr);
	if(fr_nr==1)
		str.append(" frame.");
	else
		str.append(" frames.");
	frameNrLabel->setText(str);
}

int ControlBarWidget::getFrequency()
{
	return frequency;
}

void ControlBarWidget::setFrequency(int freq)
{
	int index = 0;
	while(freq > frequencyValue[index] && index < 9)
		index++;
	frequencyComboBox->setCurrentIndex(index);
	
}

void ControlBarWidget::setStopped()
{
	setPlayPause(false);
}

//==============slots implementation==========================

void ControlBarWidget::setLoop()
{
	if(loop)
	{
		loop = false;
		loopPushButton->setIcon(QIcon(":/images/loopUP.png"	));
	}
	else
	{
		loop = true;
		loopPushButton->setIcon(QIcon(":/images/loopDOWN.png"));
	}
	emit loopStatusChanged(loop);
}

void ControlBarWidget::setPlayPause(bool play)
{
	if(play)
		playPushButton->setIcon(QIcon(":/images/pause.png"		));
	else
		playPushButton->setIcon(QIcon(":/images/play.png"		));
}

void ControlBarWidget::setFrameNrSlider(int fr_nr)
{
	if(fr_nr != current_frame_nr)
		current_frame_nr = fr_nr;
	currentFrameNrSlider->setValue(current_frame_nr);
	currentFrameNrSpinBox->setValue(current_frame_nr);
}

void ControlBarWidget::setFrameNrSpinBox(int fr_nr)
{
	if(fr_nr != current_frame_nr)
		current_frame_nr = fr_nr;
	currentFrameNrSpinBox->setValue(current_frame_nr);
	currentFrameNrSlider->setValue(current_frame_nr);
}

void ControlBarWidget::frequencySet(int index)
{
	this->frequency = frequencyValue[index];
}

bool ControlBarWidget::getLoopStatus()
{
	return loop;
}

//================= VIDEO SIZE BOX ==============================sizeX=======00

VideoSizeBox::VideoSizeBox(QWidget *parent, Qt::WFlags flags): QWidget(parent, flags)
{
	QFont f("Sans Serif", 10);
	this->setFont(f);
	setButtons();
}

void VideoSizeBox::setButtons()
{
	QCIFButton 		= new QRadioButton(tr("QCIF"	 ),this);
	CIFButton			= new QRadioButton(tr("CIF"		 ),this);
	VGAButton 		= new QRadioButton(tr("VGA"		 ),this);
	SDTVButton		= new QRadioButton(tr("SDTV"  ),this);
	HDTVButton		= new QRadioButton(tr("720p"  ),this);
  SHDTVButton    = new QRadioButton(tr("1080p"  ),this);
	customButton	= new QRadioButton(tr("Custom"),this);
	sizeXLineEdit = new QLineEdit(this);
	sizeYLineEdit	= new QLineEdit(this);
	XLabel				= new QLabel(tr("x:"),this);
	YLabel				= new QLabel(tr("y:"),this);

	QCIFButton->setGeometry (  5, 5,75,15);
	QCIFButton->setToolTip 	("176x144");
	CIFButton->setGeometry  ( 75, 5,75,15);
	CIFButton->setToolTip	("352x288");
	VGAButton->setGeometry	(145, 5,75,15);
	VGAButton->setToolTip	("640x480");
	SDTVButton->setGeometry	(  5,25,75,15);
  SDTVButton->setToolTip  ("720x576");
	HDTVButton->setGeometry ( 75,25,75,15);
  HDTVButton->setToolTip  ("1280x720");
	SHDTVButton->setGeometry (145,25,75,15);
	SHDTVButton->setToolTip 	("1920x1080");
	customButton->setGeometry(  5,45,75,15);
	XLabel->setGeometry     ( 95,45,12,15);
	sizeXLineEdit->setGeometry(110,45,40,20);
	YLabel->setGeometry     (155,45,12,15);
	sizeYLineEdit->setGeometry(170,45,40,20);
	sizeXLineEdit->setEnabled(false);
	sizeYLineEdit->setEnabled(false);

	connect(  QCIFButton, SIGNAL(clicked()), this, SLOT(setVideoSize()));
	connect(   CIFButton, SIGNAL(clicked()), this, SLOT(setVideoSize()));
	connect(  SDTVButton, SIGNAL(clicked()), this, SLOT(setVideoSize()));
	connect(   VGAButton, SIGNAL(clicked()), this, SLOT(setVideoSize()));
	connect(  HDTVButton, SIGNAL(clicked()), this, SLOT(setVideoSize()));
  connect( SHDTVButton, SIGNAL(clicked()), this, SLOT(setVideoSize()));
	connect(customButton, SIGNAL(clicked()), this, SLOT(setVideoSize()));

	CIFButton->setChecked(true);
	sizeX = CIF_X;
	sizeY = CIF_Y;

	QRegExp regExp("[1-9][0-9]{1,3}");
	sizeXLineEdit->setValidator(new QRegExpValidator(regExp, this));
	sizeYLineEdit->setValidator(new QRegExpValidator(regExp, this));
}

int VideoSizeBox::getVideoSizeX()
{
	if(customButton->isChecked())
		sizeX = sizeXLineEdit->text().toInt();
	return sizeX;
}
int VideoSizeBox::getVideoSizeY()
{
	if(customButton->isChecked())
		sizeY = sizeYLineEdit->text().toInt();
	return sizeY;
}

//==============slots implementation==========================

void VideoSizeBox::setVideoSize()
{
	if(QCIFButton->isChecked())
	{
		sizeXLineEdit->setEnabled(false);
		sizeYLineEdit->setEnabled(false);
		sizeX = QCIF_X;
		sizeY = QCIF_Y;
	}
	else if(CIFButton->isChecked())
	{
		sizeXLineEdit->setEnabled(false);
		sizeYLineEdit->setEnabled(false);
		sizeX = CIF_X;
		sizeY = CIF_Y;
	}
	else if(VGAButton->isChecked())
	{
		sizeXLineEdit->setEnabled(false);
		sizeYLineEdit->setEnabled(false);
		sizeX = VGA_X;
		sizeY = VGA_Y;
	}
	else if(SDTVButton->isChecked())
	{
		sizeXLineEdit->setEnabled(false);
		sizeYLineEdit->setEnabled(false);
		sizeX = SDTV_X;
		sizeY = SDTV_Y;
	}
	else if(HDTVButton->isChecked())
	{
		sizeXLineEdit->setEnabled(false);
		sizeYLineEdit->setEnabled(false);
		sizeX = HDTV_X;
		sizeY = HDTV_Y;
	}
  else if(SHDTVButton->isChecked())
  {
    sizeXLineEdit->setEnabled(false);
    sizeYLineEdit->setEnabled(false);
    sizeX = SHDTV_X;
    sizeY = SHDTV_Y;
  }
	else
	{
		sizeXLineEdit->setEnabled(true);
		sizeYLineEdit->setEnabled(true);
		this->setTabOrder(customButton, sizeXLineEdit);
		this->setTabOrder(sizeXLineEdit, sizeYLineEdit);
		sizeXLineEdit->setFocus();
		//sizeX and sizeY values are set in getVideoSizeX and getVideoSizeY
	}
}

int VideoSizeBox::getCurrentButton()
{
	if(QCIFButton->isChecked())
		return 0;
	else if(CIFButton->isChecked()) 
		return 1;
	else if(SDTVButton->isChecked())
		return 2;
	else if(VGAButton->isChecked())
		return 3;
	else if(HDTVButton->isChecked()) 
		return 4;
  else if(SHDTVButton->isChecked()) 
    return 5;
	else return 6;
}

void VideoSizeBox::setCurrentButton(int currentButton, QString XLine, QString YLine)
{
	if(currentButton == 0)
		QCIFButton->setChecked(true);
	else if(currentButton == 1)
		CIFButton->setChecked(true);
	else if(currentButton == 2)
		SDTVButton->setChecked(true);
	else if(currentButton == 3)
		VGAButton->setChecked(true);
	else if(currentButton == 4)
		HDTVButton->setChecked(true);
  else if(currentButton == 5)
    SHDTVButton->setChecked(true);
	else
		customButton->setChecked(true);
	sizeXLineEdit->setText(XLine);
	sizeYLineEdit->setText(YLine);
	setVideoSize();
}

QString VideoSizeBox::getQLineEditX()
{
	return sizeXLineEdit->text();
}

QString VideoSizeBox::getQLineEditY()
{
	return sizeYLineEdit->text();
}

//=======================VIDEO FORMAT BOX =======================================

VideoFormatBox::VideoFormatBox(QWidget *parent, Qt::WFlags flags): QWidget(parent, flags)
{
	QFont f("Sans Serif", 10);
	this->setFont(f);
	this->setButtons();
}

void VideoFormatBox::setButtons()
{
	format400Button	= new QRadioButton(tr("400"),this);
	format420Button	= new QRadioButton(tr("420"),this);
	format422Button	= new QRadioButton(tr("422"),this);
	format444Button	= new QRadioButton(tr("444"),this);

	format400Button->setGeometry( 5,  5,75,15);
	format420Button->setGeometry(75,  5,75,15);
	format422Button->setGeometry( 5, 25,75,15);
	format444Button->setGeometry(75, 25,75,15);

	connect(format400Button, SIGNAL(clicked()), this, SLOT(setVideoFormat()));
	connect(format420Button, SIGNAL(clicked()), this, SLOT(setVideoFormat()));
	connect(format422Button, SIGNAL(clicked()), this, SLOT(setVideoFormat()));
	connect(format444Button, SIGNAL(clicked()), this, SLOT(setVideoFormat()));

	format420Button->setChecked(true);
	videoFormat = _420;
}

int VideoFormatBox::getVideoFormat()
{
	return videoFormat;
}

int VideoFormatBox::getCurrentButton()
{
	if(format400Button->isChecked())
		return 0;
	else if(format420Button->isChecked())
		return 1;
	else if(format422Button->isChecked()) 
		return 2;
	else return 3;
}

void VideoFormatBox::setCurrentButton(int currentButton)
{
	if(currentButton == 0)
		format400Button->setChecked(true);
	else if(currentButton == 1)
		format420Button->setChecked(true);
	else if(currentButton == 2)
		format422Button->setChecked(true);
	else 
		format444Button->setChecked(true);
	setVideoFormat();
}


//==============slots implementation==========================

void VideoFormatBox::setVideoFormat()
{
	if(format400Button->isChecked())
		videoFormat = _400;
	else if(format420Button->isChecked())
		videoFormat = _420;
	else if(format422Button->isChecked())
		videoFormat = _422;
	else
		videoFormat = _444;
}

VideoSizeFormatTabWidget::VideoSizeFormatTabWidget(QWidget *parent) : QTabWidget(parent)
{
	videoFormat = new VideoFormatBox(0);
	videoSize		= new VideoSizeBox(0);
	this->addTab(videoSize, "Video Size");
	this->addTab(videoFormat, "Video Format");
}

VideoSizeFormatTabWidget::~VideoSizeFormatTabWidget()
{
	delete videoFormat;
	delete videoSize;
}

//===================== CONTROL TOOL BOX =============================================

ControlToolBox::ControlToolBox(QWidget *parent, Qt::WFlags flags): QToolBox(parent, flags)
{
	this->setWindowTitle("Control Bar");
	controlBar  = new ControlBarWidget(this);
	tabWidget   = new QWidget(this);
	videoSizeFormat = new VideoSizeFormatTabWidget(tabWidget);
	videoSizeFormat->setFixedSize(215,100);
	this->addItem(controlBar, "Control Bar");
	this->addItem(tabWidget, "Settings");
	this->setFixedSize(220,220);
	okButton = new QPushButton("OK",tabWidget);
	cancelButton = new QPushButton("Cancel",tabWidget);
	okButton->setGeometry    ( 10,120,70, 30);
	cancelButton->setGeometry(100,120,70, 30);
	connect(okButton,       SIGNAL(clicked()),              this, SLOT(okButtonClicked()));
	connect(cancelButton, 	SIGNAL(clicked()),              this, SLOT(cancelButtonClicked()));
	connect(this, 	        SIGNAL(currentChanged (int)),	this, SLOT(storeState() ));
}

ControlToolBox::~ControlToolBox()
{
	delete controlBar;
	delete tabWidget;
}

void ControlToolBox::setTotalFrameNr(int fr_nr)
{
	controlBar->setFrameNrRange(fr_nr);
	controlBar->setFrameNrLabel(fr_nr);
}

int ControlToolBox::getFormat()
{
	return videoSizeFormat->videoFormat->getVideoFormat();
}

int ControlToolBox::getSizeX()
{
	return videoSizeFormat->videoSize->getVideoSizeX();
}

int ControlToolBox::getSizeY()
{
	return videoSizeFormat->videoSize->getVideoSizeY();
}


void ControlToolBox::openEvent(int event)
{
	if(event == 1)
	{
		this->cancelButton->setDisabled(true);
		this->setItemEnabled(0, false);
		this->showSettings();
		this->showNormal();
	}
	else
		emit loadFilesEmitted();
}

void ControlToolBox::closeEvent(QCloseEvent * event )
{
	emit controlToolBoxClosed();
	event->accept();
}

void ControlToolBox::showSettings()
{
	this->setCurrentWidget(tabWidget);
}

void ControlToolBox::setCurrentFrameNr(int value)
{
	controlBar->setFrameNrSlider(value);
}

void ControlToolBox::setStopped()
{
	controlBar->setStopped();
}

void ControlToolBox::setButtons(int sX, int sY, int format, bool loop, int freq)
{
	int currentSizeButton, currentFormatButton;
	if(sX == QCIF_X && sY == QCIF_Y)
		currentSizeButton = 0;
	else if(sX == CIF_X && sY == CIF_Y)
		currentSizeButton = 1;
	else if(sX == VGA_X && sY == VGA_Y)
		currentSizeButton = 2;
	else if(sX == SDTV_X && sY == SDTV_Y)
		currentSizeButton = 3;
	else if(sX == HDTV_X && sY == HDTV_Y)
		currentSizeButton = 4;
  else if(sX == SHDTV_X && sY == SHDTV_Y)
    currentSizeButton = 5;
	else
	{
		currentSizeButton = 6;
		sizeXString.setNum(sX);
		sizeYString.setNum(sY);
	}
	currentFormatButton = format;
	videoSizeFormat->videoFormat->setCurrentButton(currentFormatButton);
	videoSizeFormat->videoSize->setCurrentButton(currentSizeButton, sizeXString, sizeYString);
	if(loop)
		controlBar->setLoop();
	controlBar->setFrequency(freq);	
}

void ControlToolBox::setPlayPause(bool play)
{
	controlBar->setPlayPause(play);
}

int ControlToolBox::getFrequency()
{
	return controlBar->getFrequency();
}

bool ControlToolBox::getLoopStatus()
{
	return controlBar->getLoopStatus();
}

//---------------slots implementation---------------------------

void ControlToolBox::okButtonClicked()
{
	okClicked = true;
	this->setCurrentWidget(controlBar);
	emit loadFilesEmitted();
	this->cancelButton->setDisabled(false);
	this->setItemEnabled(0, true);
}

void ControlToolBox::cancelButtonClicked()
{
	this->setCurrentWidget(controlBar);
	this->restoreState();
	okClicked = false;
}

void ControlToolBox::storeState()
{
	if(this->currentIndex() == 1)
	{
		currentSizeButton = videoSizeFormat->videoSize->getCurrentButton();
		if(currentSizeButton == 6)
		{
			sizeXString = videoSizeFormat->videoSize->getQLineEditX();
			sizeYString = videoSizeFormat->videoSize->getQLineEditY();
		}
		currentFormatButton = videoSizeFormat->videoFormat->getCurrentButton();
		okClicked = false;
	}
	else if(!okClicked)
		cancelButtonClicked();
}

void ControlToolBox::restoreState()
{
	videoSizeFormat->videoFormat->setCurrentButton(currentFormatButton);
	videoSizeFormat->videoSize->setCurrentButton(currentSizeButton, sizeXString, sizeYString);
}
