/***************************************************************************
 *   Copyright (C) 2005 by Yuri Vatis   																	 *
 *   vatis@tnt.uni-hannover.de																						 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <stdio.h>
#include <QMenuBar>
#include <QMenu>
#include <QAction>
#include <QFileDialog>
#include <QTimer>
#include <QApplication>
#include <QIcon>
#include <QLabel>
#include <QMoveEvent>
#include <QPushButton>
#include <QClipboard>
#include <QMimeData>
#include "mainwindow.h"

MainWindow::MainWindow(QWidget *parent, Qt::WFlags f): QMainWindow(parent, f)
{
	controlToolBox = new ControlToolBox(0, Qt::Dialog);
	controlToolBox->setWindowIcon(QIcon(":/images/icon.png"));
	seqdisplay1 = new Seqdis(this);
	setCentralWidget(seqdisplay1);
	seqdisplay2 = new Seqdis(this, Qt::Window);
	timer = new QTimer(this);
	createMenus();
	createActions();
	createSignalSlotConnections();
	initFileDialog();
	file1open = file2open = diffopen = false;
	play = false;
	file1open=file2open=false;
	difference = new Difference(this,Qt::Window);
	difference->setWindowState((difference->windowState() & ~Qt::WindowMinimized) | Qt::WindowActive);
	difference->setWindowIcon(QIcon(":/images/icon.png"));
	helpWidget = new HelpWidget(this, Qt::Window);
	helpWidget->setWindowIcon(QIcon(":/images/icon.png"));
	this->setWindowIcon(QIcon(":/images/icon.png"));
	this->setFocusPolicy(Qt::StrongFocus);
}


MainWindow::~MainWindow()
{
	delete fileDialog;
	delete controlToolBox;
}


void MainWindow::createMenus()
{
	menuBar = new QMenuBar(this);

	fileMenu = new QMenu("&File", this);
	editMenu = new QMenu("&Edit", this);
	viewMenu = new QMenu("&View", this);
	helpMenu = new QMenu("&Help", this);

	viewDiffMenu = new QMenu("View Difference", this);

	menuBar->addMenu(fileMenu);
	menuBar->addMenu(editMenu);
	menuBar->addMenu(viewMenu);
	menuBar->addMenu(helpMenu);
	this->setMenuBar(menuBar);
}

void MainWindow::createActions()
{
	open1Act = new QAction(tr("Open Video 1"),this);
	open1Act->setShortcut(tr("Ctrl+1"));
	open2Act = new QAction(tr("Open Video 2"),this);
	open2Act->setShortcut(tr("Ctrl+2"));
	open2Act->setEnabled(false);
	exitAct  = new QAction(tr("Exit"),this);
	exitAct->setShortcut(tr("Ctrl+Q"));

	fileMenu->addAction(open1Act);
	fileMenu->addAction(open2Act);
	fileMenu->addSeparator();
	fileMenu->addAction(exitAct);
	
	clip1Act = new QAction(tr("Copy 1. Image to Clipboard"),this);
	clip2Act = new QAction(tr("Copy 2. Image to Clipboard"),this);
	clipDAct = new QAction(tr("Copy Difference to Clipboard"),this);
	
	editMenu->addAction(clip1Act);
	editMenu->addAction(clip2Act);
	editMenu->addAction(clipDAct);
	clip1Act->setEnabled(false);
	clip2Act->setEnabled(false);
	clipDAct->setEnabled(false);

	viewControlToolBoxAct = new QAction(tr("Control Bar"), this);
	viewControlToolBoxAct->setShortcut(tr("Ctrl+B"));
	viewControlToolBoxAct->setCheckable(true);
	viewLinDiffAct	= new QAction(tr("linear"), this);
	viewLogDiffAct	= new QAction(tr("logarithmic"), this);
	viewLinDiffAct->setCheckable(true);
	viewLogDiffAct->setCheckable(true);


	viewMenu->addAction(viewControlToolBoxAct);
	viewMenu->addMenu(viewDiffMenu);
	viewDiffMenu->addAction(viewLinDiffAct);
	viewDiffMenu->addAction(viewLogDiffAct);
	viewDiffMenu->setEnabled(false);

	aboutYUVAct = new QAction(tr("about yuvplayer"), this);
	aboutYUVAct->setShortcut(tr("Ctrl+A"));
	helpMenu->addAction(aboutYUVAct);
}

void MainWindow::createSignalSlotConnections()
{
	connect(open1Act,  		SIGNAL(triggered()),		this, SLOT(open1()));
	connect(open2Act,  		SIGNAL(triggered()),		this, SLOT(open2())	);
	connect(exitAct,   		SIGNAL(triggered()),		this, SLOT(closeApplication()));
	connect(clip1Act,  		SIGNAL(triggered()),		this, SLOT(copyToClipboard()));
	connect(clip2Act,  		SIGNAL(triggered()),		this, SLOT(copyToClipboard()));
	connect(clipDAct,  		SIGNAL(triggered()),		this, SLOT(copyToClipboard()));
	
	connect(viewControlToolBoxAct,	SIGNAL(triggered()),		this, SLOT(toggleControlToolBoxView()));
	connect(timer, 			      SIGNAL(timeout()), 		this, SLOT(playNextFrame()));
	connect(viewLinDiffAct,		SIGNAL(triggered()),		this, SLOT(showLinDifference()));
	connect(viewLogDiffAct,		SIGNAL(triggered()),		this, SLOT(showLogDifference()));
	connect(aboutYUVAct,			SIGNAL(triggered()), 		this, SLOT(showAbout()));
	connect(controlToolBox,		SIGNAL(frequencyChanged()), 	this, SLOT(setFrequency()));
	connect(controlToolBox,		SIGNAL(loadFilesEmitted()), 	this, SLOT(loadFiles()));
	connect(controlToolBox,		SIGNAL(controlToolBoxClosed()), this, SLOT(setControlToolBoxClosed()));
	connect(controlToolBox,		SIGNAL(setLoopEmitted()), 	this, SLOT(setLoop()));
	connect(controlToolBox,		SIGNAL(showNextEmitted()), 	this, SLOT(showNextFrame()));
	connect(controlToolBox,		SIGNAL(showPreviousEmitted()),	this, SLOT(showPreviousFrame()));
	connect(controlToolBox,		SIGNAL(stopVideoEmitted()),	this, SLOT(stopVideo()));
	connect(controlToolBox,		SIGNAL(playVideoEmitted()),	this, SLOT(playVideo()));
	connect(controlToolBox,		SIGNAL(sliderEmitted(int)),	this, SLOT(showFrame(int)));
}

void MainWindow::init(int sX, int sY, bool l, bool p, int d, int format, int freq, QString f1Name, QString f2Name)
{
	QString string;
	int tmp1, tmp2;
	this->loop = l;
	this->play = p;
	file1open = true;
	clip1Act->setEnabled(true);
	if (d == -1)
	{
		this->diffopen = false;
	}
	else	if(d == 0)
	{
		this->diffopen = true;
		this->diffType	= false;
		viewLinDiffAct->setChecked(true);
		clipDAct->setEnabled(true);
	}
	else if(d == 1)
	{
		this->diffopen = true;
		this->diffType = true;
		viewLogDiffAct->setChecked(true);
		clipDAct->setEnabled(true);
	}
	this->frequency = freq;
	tmp1 = f1Name.lastIndexOf("/");
	tmp2 = f1Name.length();
	string = f1Name;
	string = string.remove(tmp1+1, tmp2-tmp1);
	this->file1Name = f1Name;
	currentDir.setPath(string);
	if(f2Name != NULL)
	{
		tmp1 = f2Name.lastIndexOf("/");
		tmp2 = f2Name.length();
		this->file2Name = f2Name;
		string = f2Name;
		file2open=true;
		viewDiffMenu->setEnabled(true);
		clip2Act->setEnabled(true);
	}
	controlToolBox->setButtons(sX, sY, format, l, freq);
	loadFiles();
	viewControlToolBoxAct->setChecked(true);
	controlToolBox->show();
  this->update();
	if(this->diffopen)
		showDifference(diffType);
	if(play)
	{
		play = false;
		playVideo();
	}
}


void MainWindow::initFileDialog()
{
	QStringList filterTypes;
	fileDialog = new QFileDialog();
	currentDir.setCurrent("");
	filterTypes << "yuv video (*.yuv)"<< "y   video (*.y)"<< "any files (*.*)";
	fileDialog->setFilters(filterTypes);
}


void MainWindow::moveControlToolBoxToMainWindow()
{
	controlToolBox->move(this->x()+(this->width()-controlToolBox->width())/2, geometry().y()+geometry().height()+4);
}

void MainWindow::sequenceClosed()
{
	viewDiffMenu->setEnabled(false);
	if(difference->isVisible())
		difference->close();
	diffopen = false;
	clip2Act->setEnabled(false);
}

bool MainWindow::isPlaying()
{
	return play;
}

//-------------- events -----------------------------------------

void MainWindow::closeEvent(QCloseEvent *)
{
	this->closeApplication();
}

void MainWindow::moveEvent(QMoveEvent *)
{
	moveControlToolBoxToMainWindow();
}

void MainWindow::resizeEvent(QResizeEvent *)
{
	moveControlToolBoxToMainWindow();
}

void MainWindow::focusInEvent ( QFocusEvent *)
{
	if(controlToolBox->isVisible())
		controlToolBox->raise();
}
//-------------- slots implementation ---------------------------
void MainWindow::open1()
{
	QStringList sl;
	fileDialog->setDirectory(currentDir);
	if(fileDialog->exec() == QDialog::Accepted)
	{
		currentDir = fileDialog->directory();
		viewControlToolBoxAct->setChecked(true);
		sl=fileDialog->selectedFiles();
		if(!sl.isEmpty())
		{
			file1Name = sl.first();
			controlToolBox->openEvent(1);
			clip1Act->setEnabled(true);
		}
	}
}


void MainWindow::open2()
{
	QStringList sl;
	fileDialog->setDirectory(currentDir);
	if(fileDialog->exec() == QDialog::Accepted)
	{
		currentDir = fileDialog->directory();
		viewDiffMenu->setEnabled(true);
		sl=fileDialog->selectedFiles();
		if(!sl.isEmpty())
		{
			file2Name = sl.first();
			file2open = true;
			controlToolBox->openEvent(2);
			clip2Act->setEnabled(true);
		}
	}
}

void MainWindow::loadFiles()
{
	int f1nr=0, f2nr=0;
	int sx = controlToolBox->getSizeX();
	int sy = controlToolBox->getSizeY();
	int vf = controlToolBox->getFormat();
	int mh;
	if(sx >= 16 && sy >= 16)
	{
		seqdisplay1->setFixedSize(sx,sy);
		f1nr = seqdisplay1->loadFile(file1Name, sx, sy, vf);
	}
	if(f1nr > 0)
	{
		file1open = true;
		seqdisplay1->setFixedSize(sx, sy);
		seqdisplay1->show();
		open2Act->setEnabled(true);
		if(file2open)
			f2nr = seqdisplay2->loadFile(file2Name, sx, sy, vf);
		if(f2nr > 0)
		{
			file2open = true;
			seqdisplay2->setFixedSize(sx, sy);
			seqdisplay2->show();
			difference->setSizeXY(sx, sy);
			if(diffopen)
				showDifference(diffType);
		}
		else
		{
			viewDiffMenu->setEnabled(false);
			file2open = false;
			diffopen = false;
			clip2Act->setEnabled(false);
			clipDAct->setEnabled(false);
		}
		controlToolBox->setTotalFrameNr(f1nr);
		mh = this->menuBar->height();
		this->setFixedSize(sx, sy+mh);
		mh = this->menuBar->height();
		this->setFixedSize(sx, sy+mh);		/* do not delete! it seems to be a Qt-bug, if yuvplayer is called from command line, it first gets wrong this->menuBar->height()*/
	}
	else
	{
		file1open = false;
		file2open = false;
		open2Act->setEnabled(false);
		viewDiffMenu->setEnabled(false);
		diffopen = false;
		clip1Act->setEnabled(false);
		clip2Act->setEnabled(false);
		clipDAct->setEnabled(false);
		controlToolBox->setTotalFrameNr(0);
		if(seqdisplay2->isVisible())
			seqdisplay2->close();
		if(difference->isVisible())
			difference->close();
		this->setFixedSize(QCIF_X, QCIF_Y);
		this->update();
		seqdisplay1->close();
	}
	controlToolBox->setCurrentFrameNr(seqdisplay1->getCurrentFrameNr());
	if(difference->isVisible())
	{
		difference->update();
		difference->raise();
	}
}

void MainWindow::copyToClipboard()
{
	QClipboard *clipboard = QApplication::clipboard();
	if(this->sender() == clip1Act)
	{
	  clipboard->setImage(this->seqdisplay1->getImage(), QClipboard::Clipboard);
	}
	else if(this->sender() == clip2Act)
	{
		clipboard->setImage(seqdisplay2->getImage(), QClipboard::Clipboard);
	}
	else
	{
		clipboard->setImage(difference->getImage(), QClipboard::Clipboard);
	}

}

void MainWindow::setControlToolBoxClosed()
{
	viewControlToolBoxAct->setChecked(false);
}

void MainWindow::showNextFrame()
{
	if(play)
		controlToolBox->setStopped();
	play = false;
	timer->stop();
	seqdisplay1->showNextFrame();
	seqdisplay2->showNextFrame();
	if(diffopen)
		showDifference(diffType);
	controlToolBox->setCurrentFrameNr(seqdisplay1->getCurrentFrameNr());
}

void MainWindow::playNextFrame()
{
	if(!loop && seqdisplay1->getCurrentFrameNr() == seqdisplay1->getTotalFrameNr())
	{
		timer->stop();
		controlToolBox->setStopped();
		play = false;
	}
	if(seqdisplay1->getCurrentFrameNr() == seqdisplay1->getTotalFrameNr())
	{
		seqdisplay1->showFirstFrame();
		if(file2open)
			seqdisplay2->showFirstFrame();
	}
	else
	{
		seqdisplay1->showNextFrame();
		seqdisplay2->showNextFrame();		
	}
	if(diffopen)
		showDifference(diffType);
	controlToolBox->setCurrentFrameNr(seqdisplay1->getCurrentFrameNr());
}

void MainWindow::showPreviousFrame()
{
	//	this->setActiveWindow();
	if(play)
		controlToolBox->setStopped();
	play = false;
	timer->stop();
//	stop = 1;
	seqdisplay1->showPreviousFrame();
	seqdisplay2->showPreviousFrame();
	if(diffopen)
		showDifference(diffType);
	controlToolBox->setCurrentFrameNr(seqdisplay1->getCurrentFrameNr());
}

void MainWindow::showFrame(int fr_nr)
{
	seqdisplay1->showFrameNr(fr_nr);
	if(file2open)
		seqdisplay2->showFrameNr(fr_nr);
	if(diffopen)
		showDifference(diffType);

}

void MainWindow::playVideo()
{
	if(file1open)
	{
		if(play)
		{
			play = false;
			timer->stop();
		}
		else
		{
			this->loop = controlToolBox->getLoopStatus();
			this->frequency = controlToolBox->getFrequency();
			play = true;
			timer->start(1000/this->frequency);
		}
		controlToolBox->setPlayPause(play);
	}
	else
		controlToolBox->setStopped();
}

void MainWindow::stopVideo()
{
	play = false;
	timer->stop();
	controlToolBox->setStopped();
	if(file1open)
		seqdisplay1->showFirstFrame();
	if(file2open)
		seqdisplay2->showFirstFrame();
	controlToolBox->setCurrentFrameNr(seqdisplay1->getCurrentFrameNr());
}

void MainWindow::pauseVideo()
{
	play = false;
	timer->stop();
	controlToolBox->setStopped();
}


void MainWindow::setFrequency()
{
	this->frequency = controlToolBox->getFrequency();
	if(play)
	{
		timer->stop();
		timer->start(1000/frequency);
	}
}

void MainWindow::setLoop()
{
	this->loop = !this->loop;
}

void MainWindow::closeApplication()
{
	QCoreApplication::quit();
}

void MainWindow::toggleControlToolBoxView()
{
	if(viewControlToolBoxAct->isChecked())
		controlToolBox->showNormal();
	else
		controlToolBox->close();
}

void MainWindow::showLinDifference()
{
	diffopen = true;
	diffType = false;
	viewLinDiffAct->setChecked(true);
	viewLogDiffAct->setChecked(false);
	showDifference(diffType);
	clipDAct->setEnabled(true);
}

void MainWindow::showLogDifference()
{
	diffopen = true;
	diffType = true;
	viewLinDiffAct->setChecked(false);
	viewLogDiffAct->setChecked(true);
	showDifference(diffType);
	clipDAct->setEnabled(true);
}

void MainWindow::showDifference(bool type)
{
	difference->setFramesPointer(seqdisplay1->getYUVBuffer(), seqdisplay2->getYUVBuffer());
	difference->showDifference(type);
	difference->raise();
	if(difference->isMinimized())
		difference->showMaximized();
}

void MainWindow::diffClosed()
{
	diffopen = false;
	viewLinDiffAct->setChecked(false);
	viewLogDiffAct->setChecked(false);
	clipDAct->setEnabled(false);
}

void MainWindow::showYBlocks(QMouseEvent *event)
{
	seqdisplay1->mouseReleaseEvent(event);
	seqdisplay2->mouseReleaseEvent(event);
}

void MainWindow::showAbout()
{
	helpWidget->showNormal();
	helpWidget->raise();
}

HelpWidget::HelpWidget(QWidget *parent, Qt::WFlags f): QWidget(parent, f)
{
	QFont font("Arial", 10, QFont::Normal);
	font.setPixelSize(12);
	this->setFixedSize(400, 300);
	this->setWindowTitle(tr("About yuvplayer"));
	borderX = 10;
	borderY = 50;
	tabWidget 		= new QTabWidget (this);
	tabWidget->setGeometry(borderX, borderY, CIF_X-2*borderX, CIF_Y-2*borderY);
	aboutWidget		= new QWidget();
	authorWidget	= new QWidget();
	thankstoWidget= new QWidget();
	licenceWidget = new QWidget();
	tabWidget->addTab(aboutWidget, 	tr("About"));
	tabWidget->addTab(authorWidget, tr("Authors"));
	tabWidget->addTab(thankstoWidget, tr("Thanks to"));
	tabWidget->addTab(licenceWidget, tr("Licence"));
	closeButton = new QPushButton(tr("Close"), this);
	buttonsizeX = 100;
	buttonsizeY = 26;
	closeButton->setGeometry(this->width()-borderX-buttonsizeX,this->height()-borderY+borderX,buttonsizeX,buttonsizeY);
	picLabel = new QLabel(this);
	picLabel->setGeometry(borderX,borderX,30,30);
	picLabel->setPixmap(QPixmap(":/images/icon.png"));
	nameLabel = new QLabel("<h2> yuvplayer 1.5 </h2>",this);
	nameLabel->setGeometry(borderX+picLabel->width(), borderX, tabWidget->width(), picLabel->height());
	aboutLabel = new QLabel(aboutWidget);
	aboutLabel->setMargin(10);
	aboutLabel->setFont(font);
	aboutLabel->setText(tr("yuvplayer is a small application that shows video <br>sequences in .y and .yuv format.<br><br>Copyright &copy; 2005 <br>Yuri Vatis<br>Institut f&uuml;r Informationsverarbeitung<br>Leibniz Universit&auml;t Hannover<br><a href = http://www.tnt.uni-hannover.de/~vatis>http://www.tnt.uni-hannover.de/~vatis</a>"));
	aboutLabel->setTextFormat(Qt::RichText);
	authorLabel = new QLabel(authorWidget);
	authorLabel->setMargin(10);
	authorLabel->setFont(font);
	authorLabel->setText(tr("<p>Yuri Vatis<br><a href = mailto:vatis@tnt.uni-hannover.de>vatis@tnt.uni-hannover.de</a>"));
	authorLabel->setTextFormat(Qt::RichText);
	thankstoLabel = new QLabel(thankstoWidget);
	thankstoLabel->setMargin(10);
	thankstoLabel->setFont(font);
	thankstoLabel->setText(tr("<p>Tobias Elbrandt<br><a href = mailto:elbrandt@tnt.uni-hannover.de>elbrandt@tnt.uni-hannover.de</a> <p>Nikolaus Meine<br><a href = mailto:meine@tnt.uni-hannover.de>meine@tnt.uni-hannover.de</a> <p>Torsten Wiebesiek<br><a href = mailto:wiebesiek@tnt.uni-hannover.de>wiebesiek@tnt.uni-hannover.de</a>"));
	thankstoLabel->setTextFormat(Qt::RichText);
	licenceLabel = new QLabel(licenceWidget);
	licenceLabel->setMargin(10);
	licenceLabel->setFont(font);
	licenceLabel->setText(tr("&copy; 2005, Yuri Vatis<br><br>This program is distributed under the terms of the GPL.<br>See also <a href =http://www.gnu.org/licenses/licenses.html>http://www.gnu.org/licenses/licenses.html</a>"));
	licenceLabel->setTextFormat(Qt::RichText);
	connect(closeButton, SIGNAL(clicked()), this, SLOT(closeButtonClicked()) );
	aboutLabel	->setOpenExternalLinks(true);
	nameLabel	->setOpenExternalLinks(true);
	authorLabel	->setOpenExternalLinks(true);
	thankstoLabel	->setOpenExternalLinks(true);
	licenceLabel	->setOpenExternalLinks(true);
}

void HelpWidget::resizeEvent(QResizeEvent *)
{
	tabWidget->setGeometry(borderX, borderY, this->width()-2*borderX, this->height()-2*borderY);
	closeButton->setGeometry(this->width()-borderX-buttonsizeX,this->height()-borderY+borderX,buttonsizeX,buttonsizeY);

}
//------------------------- slots --------------------------------------
void HelpWidget::closeButtonClicked()
{
	this->close();
}
